package me.evilpeanut;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.bukkit.Location;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;

/**
 * @author Reece Aaron Lecrivain
 */
public class PlayerProfile {
	public String name, nameColor, nameTitle, nameAlias, lastMessage, teleportantName, customRankPrefix = "0[6Custom0]", customRankColor = "6";
	public List<EvilEditBlock> EvilEditUndo = new ArrayList<>(), EvilEditCopy = new ArrayList<>();
	public Location deathLocation, homeLocation, actionLocationA, actionLocationB;
	public int runAmplifier = 4, survivalXP = 0, strengthLevel = 0, miningLevel = 0, fishingLevel = 0;
	
	private int money = 0;
	public void chargeMoney(int money) {
		this.money -= money;
        plugin.getServer().getPluginManager().callEvent(new PlayerMoneyChangeEvent(plugin.getServer().getPlayer(name)));
	}
	public void addMoney(int money) {
		this.money += money;
		plugin.getServer().getPluginManager().callEvent(new PlayerMoneyChangeEvent(plugin.getServer().getPlayer(name)));
	}
	public void setMoney(int money) {
		this.money = money;
		plugin.getServer().getPluginManager().callEvent(new PlayerMoneyChangeEvent(plugin.getServer().getPlayer(name)));
	}
	public int getMoney() {return money;}
	
	public List<String> warps = new ArrayList<>(), achievements = new ArrayList<>(), mutedPlayers = new ArrayList<>(), allowedPrivateWorlds = new ArrayList<>();
	public Boolean isLogging = false, isInvisible = false, isAway = false, isAllowedNewWorld = false;
	public List<Spell> spellBook = new ArrayList<>();
	public long lastMessageTime = 0, lastActionTime = 0;
	public double jumpAmplifier = 0;
	public float flyAmplifier = 0.1f;
	public Rank rank = Rank.Builder;
	public Spell selectedSpell;
	private EvilBook plugin;
	public Entity disguise;
	
	/**
	 * Load the profile of an online player
	 * @param plugin The parent EvilBook plugin
	 * @param playerName The name of the player
	 */
	public PlayerProfile(EvilBook plugin, Player newPlayer) {
		this.plugin = plugin;
		name = newPlayer.getName();
		File file = new File("plugins/EvilBook/Players/" + name + ".properties");
		if (file.exists()) {
			Properties prop = new Properties(file);
			rank = Rank.valueOf(prop.getProperty("Rank"));
			if (rank.getID() >= Rank.Custom.getID()) {
				String prefix = prop.getProperty("CustomRankPrefix");
				if (prefix != null) {
					customRankColor = prefix.substring(1, 2);
					customRankPrefix = "0[" + prefix.replaceAll("&", "") + "0]";
				}
			}
			if (prop.getProperty("AllowedNewWorld") != null) isAllowedNewWorld = Boolean.valueOf(prop.getProperty("AllowedNewWorld"));
			if (prop.getProperty("Money") != null) {
				money = Integer.valueOf(prop.getProperty("Money"));
			} else {
				plugin.alertOwner(name + " failed to load money");
			}
			if (prop.getProperty("Spells") != null) {
				String[] spell = prop.getProperty("Spells").split(">");
				for (String s : spell) spellBook.add(Spell.valueOf(s));
				selectedSpell = spellBook.get(0);
			}
			if (prop.getProperty("HomeLocation") != null) {
				String[] location = prop.getProperty("HomeLocation").split(">");
				homeLocation = new Location(plugin.getServer().getWorld(location[3]), Double.valueOf(location[0]), Double.valueOf(location[1]), Double.valueOf(location[2]));
			}
			if (prop.getProperty("Warps") != null) warps.addAll(Arrays.asList(prop.getProperty("Warps").toLowerCase().split(",")));
			if (prop.getProperty("AllowedPrivateWorlds") != null) allowedPrivateWorlds.addAll(Arrays.asList(prop.getProperty("AllowedPrivateWorlds").toLowerCase().split(",")));
			if (prop.getProperty("Achievements") != null) achievements.addAll(Arrays.asList(prop.getProperty("Achievements").toLowerCase().split(",")));
			if (prop.getProperty("RunAmplifier") != null) runAmplifier = Integer.valueOf(prop.getProperty("RunAmplifier"));
			if (prop.getProperty("FlyAmplifier") != null) {
				flyAmplifier = Float.valueOf(prop.getProperty("FlyAmplifier"));
				newPlayer.setFlySpeed(flyAmplifier);
			}
			if (prop.getProperty("JumpAmplifier") != null) jumpAmplifier = Double.valueOf(prop.getProperty("JumpAmplifier"));
			if (prop.getProperty("SurvivalXP") != null) survivalXP = Integer.valueOf(prop.getProperty("SurvivalXP"));
			if (rank.isAdmin()) {
				nameColor = prop.getProperty("NameColor");
				nameTitle = prop.getProperty("NameTitle");
				nameAlias = prop.getProperty("NameAlias");
				if (nameAlias != null) {
					if (nameColor.equals("?")) {
						if (nameTitle == null) {
							newPlayer.setDisplayName(plugin.colorizeString(nameAlias) + "f");
						} else {
							newPlayer.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(nameAlias) + "f");
						}
					} else {
						if (nameTitle == null) {
							newPlayer.setDisplayName("" + nameColor + nameAlias + "f");
						} else {
							newPlayer.setDisplayName("d" + nameTitle + " " + nameColor + nameAlias + "f");
						}
					}
					newPlayer.setPlayerListName("" + rank.getColor(this) + (nameAlias.length() > 14 ? nameAlias.substring(0, 14) : nameAlias));
				} else {
					if (nameColor.equals("?")) {
						if (nameTitle == null) {
							newPlayer.setDisplayName(plugin.colorizeString(name) + "f");
						} else {
							newPlayer.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(name) + "f");
						}
					} else {
						if (nameTitle == null) {
							newPlayer.setDisplayName("" + nameColor + name + "f");
						} else {
							newPlayer.setDisplayName("d" + nameTitle + " " + nameColor + name + "f");
						}
					}
					newPlayer.setPlayerListName("" + rank.getColor(this) + (name.length() > 14 ? name.substring(0, 14) : name));
				}
			} else {
				newPlayer.setDisplayName("f" + name);
				newPlayer.setPlayerListName("" + rank.getColor(this) + (name.length() > 14 ? name.substring(0, 14) : name));
			}
			mutedPlayers.addAll(Arrays.asList(prop.getProperty("MutedPlayers").split(",")));
			for (Player p : plugin.getServer().getOnlinePlayers()) {
				if (p.getName() == name) continue;
				p.sendMessage("9[6" + plugin.getServer().getOnlinePlayers().length + "/" + plugin.getServer().getMaxPlayers() + "9] 6Everyone welcome " + newPlayer.getDisplayName() + "6 back to the game!");
			}
		} else {
			try {
				file.createNewFile();
				Properties prop = new Properties(file);
				prop.setProperty("Rank", "Visitor");
				rank = Rank.Builder;
				newPlayer.setPlayerListName("" + rank.getColor(this) + (name.length() > 14 ? name.substring(0, 14) : name));
				prop.setProperty("Money", "0");
				money = 0;
				prop.setProperty("NameColor", "f");
				nameColor = "f";
				prop.setProperty("MutedPlayers", "");
				newPlayer.setDisplayName("f" + name);
				prop.setProperty("ChallengeRankUp", "0");
				prop.save();
				newPlayer.getInventory().addItem(plugin.getBook("Welcome to Amentrix", "EvilPeanut", Arrays.asList("Welcome to the Amentrix Server\n\nRules:\na[b1a] 6Dont Greif\na[b2a] 6Dont Advertise\na[b3a] 6Dont Spam\n\n0Website:\n7http://amentrix.no-ip.org\n\n0Enjoy your stay!\n7 - EvilPeanut")));
			} catch (Exception exception) {
				plugin.logSevere("Failed to create " + name + "'s player profile");
				exception.printStackTrace();
			}
			for (Player p : plugin.getServer().getOnlinePlayers()) {
				if (p.getName() == name) continue;
				p.sendMessage("d" + new File("plugins/EvilBook/Players").list().length + " 9players have joined the server");
				p.sendMessage("9[6" + plugin.getServer().getOnlinePlayers().length + "/" + plugin.getServer().getMaxPlayers() + "9] 6Everyone welcome " + newPlayer.getDisplayName() + "6 for the first time!");
			}
		}
		newPlayer.sendMessage("2Welcome to Amentrix");
		if (rank.isAdmin() == false) {
			newPlayer.sendMessage("3Type 6/admin 3to discover how to become an admin");
			newPlayer.sendMessage("3Type 6/survival 3to enter the survival world");
		} else {
			newPlayer.sendMessage("3Type 6/shop 3to browse the shop catalogue");
		}
		newPlayer.sendMessage("3Type 6/ranks 3for the list of ranks");
		newPlayer.sendMessage("3Type 6/donate 3for instructions on how to donate");
		newPlayer.sendMessage("3Type 6/help 3for help");
	}
	
	/**
	 * Save the player profile
	 */
	public void saveProfile() {
		Properties prop = new Properties("plugins/EvilBook/Players/" + name + ".properties");
		// Player profile properties to save
		if (homeLocation != null) prop.setProperty("HomeLocation", homeLocation.getX() + ">" + homeLocation.getY() + ">" + homeLocation.getZ() + ">" + homeLocation.getWorld().getName());
		if (nameColor != null) prop.setProperty("NameColor", nameColor);
		if (nameTitle != null) prop.setProperty("NameTitle", nameTitle);
		if (nameAlias != null) prop.setProperty("NameAlias", nameAlias);
		if (mutedPlayers != null) prop.setProperty("MutedPlayers", StringUtils.join(mutedPlayers, ","));
		prop.setProperty("Rank", rank.toString());
		prop.setProperty("Money", Integer.toString(money));
		if (warps != null) prop.setProperty("Warps", StringUtils.join(warps, ","));
		if (allowedPrivateWorlds != null) prop.setProperty("AllowedPrivateWorlds", StringUtils.join(allowedPrivateWorlds, ","));
		prop.setProperty("RunAmplifier", Integer.toString(runAmplifier));
		prop.setProperty("FlyAmplifier", Float.toString(flyAmplifier));
		prop.setProperty("JumpAmplifier", Double.toString(jumpAmplifier));
		prop.setProperty("SurvivalXP", Integer.toString(survivalXP));
		prop.setProperty("AllowedNewWorld", Boolean.toString(isAllowedNewWorld));
		//
		prop.save();
	}
	
	/**
	 * Set the name color of the player
	 * @param color The name color
	 * @param player The player
	 */
	public void setNameColor(String color, Player player) {
		setProperty("NameColor", color);
		nameColor = color;
		if (nameAlias == null) {
			if (nameColor.equals("?")) {
				if (nameTitle == null) {
					player.setDisplayName(plugin.colorizeString(name) + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(name) + "f");
				}
			} else {
				if (nameTitle == null) {
					player.setDisplayName("" + nameColor + name + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + nameColor + name + "f");
				}
			}
		} else {
			if (nameColor.equals("?")) {
				if (nameTitle == null) {
					player.setDisplayName(plugin.colorizeString(nameAlias) + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(nameAlias) + "f");
				}
			} else {
				if (nameTitle == null) {
					player.setDisplayName("" + nameColor + nameAlias + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + nameColor + nameAlias + "f");
				}
			}
		}
	}
	
	/**
	 * Set the title of the player
	 * @param title The title
	 * @param player The player
	 */
	public void setNameTitle(String title, Player player) {
		setProperty("NameTitle", title);
		nameTitle = title;
		if (nameAlias == null) {
			if (nameColor.equals("?")) {
				if (nameTitle == null) {
					player.setDisplayName(plugin.colorizeString(name) + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(name) + "f");
				}
			} else {
				if (nameTitle == null) {
					player.setDisplayName("" + nameColor + name + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + nameColor + name + "f");
				}
			}
		} else {
			if (nameColor.equals("?")) {
				if (nameTitle == null) {
					player.setDisplayName(plugin.colorizeString(nameAlias) + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(nameAlias) + "f");
				}
			} else {
				if (nameTitle == null) {
					player.setDisplayName("" + nameColor + nameAlias + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + nameColor + nameAlias + "f");
				}
			}
		}
	}

	/**
	 * Set the name alias of the player
	 * @param alias The name alias
	 * @param player The player
	 */
	public void setNameAlias(String alias, Player player) {
		if (alias == null) {
			removeProperty("NameAlias");
			nameAlias = null;
			if (nameColor.equals("?")) {
				if (nameTitle == null) {
					player.setDisplayName(plugin.colorizeString(name) + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(name) + "f");
				}
			} else {
				if (nameTitle == null) {
					player.setDisplayName("" + nameColor + name + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + nameColor + name + "f");
				}
			}
			player.setPlayerListName("" + rank.getColor(this) + player.getName());
		} else {
			setProperty("NameAlias", alias);
			nameAlias = alias;
			if (nameColor.equals("?")) {
				if (nameTitle == null) {
					player.setDisplayName(plugin.colorizeString(nameAlias) + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + plugin.colorizeString(nameAlias) + "f");
				}
			} else {
				if (nameTitle == null) {
					player.setDisplayName("" + nameColor + nameAlias + "f");
				} else {
					player.setDisplayName("d" + nameTitle + " " + nameColor + nameAlias + "f");
				}
			}
			player.setPlayerListName("" + rank.getColor(this) + nameAlias);
		}
	}

	/**
	 * Set a property on the player profile file
	 * @param property The property to set
	 * @param value The value of the property
	 */
	public void setProperty(String property, String value) {
		File file = new File("plugins/EvilBook/Players/" + name + ".properties");
		if (file.exists()) {
			Properties prop = new Properties(file);
			prop.setProperty(property, value);
			prop.save();
		}
	}
	
	/**
	 * Remove a property on the player profile file
	 * @param property The property to remove
	 */
	public void removeProperty(String property) {
		File file = new File("plugins/EvilBook/Players/" + name + ".properties");
		if (file.exists()) {
			Properties prop = new Properties(file);
			prop.removeProperty(property);
			prop.save();
		}
	}
	
	/**
	 * Add a spell to the player profile
	 * @param spell The spell to add
	 */
	public void addSpell(Spell spell) {
		spellBook.add(spell);
		Properties prop = new Properties("plugins/EvilBook/Players/" + name + ".properties");
		// Player profile properties to save
		if (spellBook.size() == 1) {
			prop.setProperty("Spells", spell.toString());
		} else {
			String spells = "";
			for (Spell s : spellBook) spells += spells == "" ? s.toString() : ">" + s.toString();
			prop.setProperty("Spells", spells);
		}
		//
		prop.save();
	}
	
	/**
	 * Set the player's last position in the world
	 * Use when they teleport to a different world
	 * @param location Their location in the world
	 */
	public void setWorldLastPosition(Location location) {
		Properties playerProperties = new Properties("plugins/EvilBook/Players/" + name + ".properties");
		playerProperties.setProperty(location.getWorld().getName() + "Location", location.getX() + ">" + location.getY() + ">" + location.getZ());
		playerProperties.save();
	}
	
	/**
	 * Get the player's last position in the world
	 * @param world The world to get their last location in
	 */
	public Location getWorldLastPosition(String world) {
		Properties playerProperties = new Properties("plugins/EvilBook/Players/" + name + ".properties");
		if (playerProperties.getProperty(world + "Location") != null) {
			String[] location = playerProperties.getProperty(world + "Location").split(">");
			return new Location(plugin.getServer().getWorld(world), Double.valueOf(location[0]), Double.valueOf(location[1]), Double.valueOf(location[2]));
		}
		return plugin.getServer().getWorld(world).getSpawnLocation();
	}
	
	public void updatePlayerListName(Player player) {
		if (nameAlias != null) {
			if (isAway) {
				player.setPlayerListName("7*" + rank.getColor(this) + (nameAlias.length() > 11 ? nameAlias.substring(0, 11) : nameAlias));
			} else {
				player.setPlayerListName("" + rank.getColor(this) + (nameAlias.length() > 14 ? nameAlias.substring(0, 14) : nameAlias));
			}
		} else {
			if (isAway) {
				player.setPlayerListName("7*" + rank.getColor(this) + (name.length() > 11 ? name.substring(0, 11) : name));
			} else {
				player.setPlayerListName("" + rank.getColor(this) + (name.length() > 14 ? name.substring(0, 14) : name));
			}
		}
	}
}
