package me.evilpeanut;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.bukkit.Location;

/**
 * @author Reece Aaron Lecrivain
 */
public class Region {
	private Boolean isProtected = false;
	private String regionName, welcomeMessage = null, leaveMessage = null, warpName = null, ownerName;
	private Location locationA, locationB;
	private List<String> allowedPlayers = new ArrayList<>();
	
	/**
	 * @return The players who have rights to the region
	 */
	List<String> getAllowedPlayers(){ return allowedPlayers; }
	
	/**
	 * @param allowedPlayers The players who have rights to the region
	 */
	void setAllowedPlayers(List<String> allowedPlayers) { this.allowedPlayers = allowedPlayers; }
	
	/**
	 * @param playerName The player to add to the region permissions
	 */
	void addAllowedPlayer(String playerName) { if (allowedPlayers.contains(playerName) == false) allowedPlayers.add(playerName); }
	
	/**
	 * @param playerName The player to remove region permissions of
	 */
	void removeAllowedPlayer(String playerName) { if (allowedPlayers.contains(playerName)) allowedPlayers.remove(playerName); }
	
	/**
	 * @param playerName The player to check for region permissions
	 */
	Boolean isPermitted(String playerName) { return allowedPlayers.contains(playerName) ? true : false; }
	
	/**
	 * @return The region owner's name
	 */
	String getOwner(){ return ownerName; }
	
	/**
	 * @param ownerName The region owner's name
	 */
	void setOwner(String ownerName) { this.ownerName = ownerName; }
	
	/**
	 * @return The warp redirection name
	 */
	String getWarp(){ return warpName; }
	
	/**
	 * @param warpName The warp redirection name
	 */
	void setWarp(String warpName) { this.warpName = warpName; }
	
	/**
	 * @return The leave message
	 */
	String getLeaveMessage(){ return leaveMessage; }
	
	/**
	 * @param leaveMessage The leave message
	 */
	void setLeaveMessage(String leaveMessage) { this.leaveMessage = leaveMessage; }
	
	/**
	 * @return The welcome message
	 */
	String getWelcomeMessage(){ return welcomeMessage; }
	
	/**
	 * @param welcomeMessage The welcome message
	 */
	void setWelcomeMessage(String welcomeMessage) { this.welcomeMessage = welcomeMessage; }
	
	/**
	 * @return The region nam
	 */
	String getRegionName(){ return regionName; }
	
	/**
	 * @param regionName The region name
	 */
	void setRegionName(String regionName) { this.regionName = regionName; }
	
	/**
	 * @return If protection is enabled
	 */
	Boolean isProtected(){ return isProtected; }
	
	/**
	 * @param isProtected If protection is enabled
	 */
	void isProtected(Boolean isProtected) { this.isProtected = isProtected; }
	
	/**
	 * @return The first location point
	 */
	Location getLocationA(){ return locationA; }
	
	/**
	 * @param locationA The first location point
	 */
	void setLocationA(Location locationA) { this.locationA = locationA; }
	
	/**
	 * @return The second location point
	 */
	Location getLocationB(){ return locationB; }
	
	/**
	 * @param locationB The second location point
	 */
	void setLocationB(Location locationB) { this.locationB = locationB; }

	/**
	 * Define a new region
	 * @param plugin The EvilBook parent
	 * @param regionName The name of the region
	 * @param locationA The first location point of the region
	 * @param locationB The second location point of the region
	 * @param isProtected If protection is enabled in the region
	 * @param ownerName The region owner's name
	 * @param welcomeMessage The welcome message of the region
	 * @param leaveMessage The leave message of the region
	 * @param allowedPlayers The players who have rights to the region
	 */
	public Region(EvilBook plugin, String regionName, Location locationA, Location locationB, Boolean isProtected, String ownerName, String welcomeMessage, String leaveMessage, String allowedPlayers, String warpName)
	{
		this.regionName = regionName;
		this.locationA = locationA;
		this.locationB = locationB;
		this.isProtected = isProtected;
		this.ownerName = ownerName;
		if (welcomeMessage != null) this.welcomeMessage = plugin.toFormattedString(welcomeMessage);
		if (leaveMessage != null) this.leaveMessage = plugin.toFormattedString(leaveMessage);
		if (allowedPlayers != null) this.allowedPlayers = Arrays.asList(allowedPlayers.split(","));
		if (warpName != null) this.warpName = warpName;
	}

	/**
	 * Save the region externally
	 */
	public void saveRegion() 
	{
		Properties regionProp = new Properties();
		regionProp.setProperty("isProtected", isProtected.toString());
		regionProp.setProperty("worldName", locationA.getWorld().getName());
		regionProp.setProperty("locationA", locationA.getBlockX() + "," + locationA.getBlockY() + "," + locationA.getBlockZ());
		regionProp.setProperty("locationB", locationB.getBlockX() + "," + locationB.getBlockY() + "," + locationB.getBlockZ());
		regionProp.setProperty("ownerName", ownerName);
		if (welcomeMessage != null) regionProp.setProperty("welcomeMessage", welcomeMessage);
		if (leaveMessage != null) regionProp.setProperty("leaveMessage", leaveMessage);
		if (allowedPlayers != null) regionProp.setProperty("allowedPlayers", StringUtils.join(allowedPlayers, ","));
		if (warpName != null) regionProp.setProperty("warpName", warpName);
		regionProp.save("plugins/EvilBook/Regions/" + regionName + ".properties", null);
	}
}