package me.evilpeanut;

/**
 * @author Reece Aaron Lecrivain
 */
public enum Rank {
	Builder((byte)0, "Builder", "0[EBuilder0]", "E", 0),
	AdvancedBuilder((byte)1, "Advanced Builder", "0[5Adv.Builder0]", "5", 0),
	Architect((byte)2, "Architect", "0[DArchitect0]", "D", 0),
	Moderator((byte)3, "Moderator", "0[9Moderator0]", "9", 0),
	Police((byte)4, "Police", "0[3Police0]", "3", 0),
	Admin((byte)5, "Admin", "0[4Admin0]", "4", 5000),
	SpecialAdmin((byte)6, "Special Admin", "0[4SpecAdmin0]", "4", 10000),
	Councillor((byte)7, "Councillor", "0[ACouncillor0]", "A", 15000),
	Elite((byte)8, "Elite", "0[CElite0]", "C", 20000),
	CoOwner((byte)9, "Co-Owner", "0[6Co-Owner0]", "6", 22000),
	Owner((byte)10, "Owner", "0[BOwner0]", "B", 25000),
	RainbowOwner((byte)11, "Rainbow Owner", "0[AOBwCnDeEr0]", "A", 50000),
	Custom((byte)12, "Custom Rank", "0[6Custom0]", "6", 75000),
	ServerOwner((byte)13, "Server Owner", "0[BServerInvestor0]", "B", 100000),
	ServerHost((byte)14, "Server Host", "0[BServerHost0]", "B", 10000000);
	
	private Byte ID;
	private String prefix, color, name;
	private Integer evilEditAreaLimit;
	
	/**
	 * @return If the player is an admin
	 */
	Boolean isAdmin() { return ID >= Rank.Admin.getID() ? true : false; }
	
	/**
	 * @return The ID of the rank
	 */
	Byte getID(){ return ID; }
	
	/**
	 * @return The name of the rank
	 */
	String getName(){ return name; }
	
	/**
	 * @return The prefix of the rank
	 */
	String getPrefix(PlayerProfile player) { 
		if (player.rank.ID >= Rank.Custom.getID()) {
			return player.customRankPrefix;
		} else {
			return prefix; 
		}
	}
	
	/**
	 * @return The color of the rank
	 */
	String getColor(PlayerProfile player) {
		if (player.rank.ID >= Rank.Custom.getID()) {
			return player.customRankColor;
		} else {
			return color; 
		}
	}
	
	/**
	 * @return The evil edit area limit of the rank
	 */
	Integer getEvilEditAreaLimit(){ return evilEditAreaLimit; }
	
	/**
	 * Define a new rank
	 * @param ID The ID of the rank
	 * @param name The name of the rank
	 * @param prefix The prefix of the rank
	 * @param color The color of the rank
	 * @param evilEditAreaLimit The evil edit area limit of the rank
	 */
	Rank (Byte ID, String name, String prefix, String color, Integer evilEditAreaLimit) {
		this.ID = ID;
		this.name = name;
		this.prefix = prefix;
		this.color = color;
		this.evilEditAreaLimit = evilEditAreaLimit;
	}
	
	/**
	 * Return the next rank on the rank ladder
	 * @param rank The rank
	 * @return The next rank on the rank ladder
	 */
	public static Rank getNextRank(Rank rank) {
		switch (rank) {
		case Builder:
			return Rank.AdvancedBuilder;
		case AdvancedBuilder:
			return Rank.Architect;
		case Architect:
			return Rank.Moderator;
		case Moderator:
			return Rank.Police;
		case Police:
			return Rank.Admin;
		case Admin:
			return Rank.SpecialAdmin;
		case SpecialAdmin:
			return Rank.Councillor;
		case Councillor:
			return Rank.Elite;
		case Elite:
			return Rank.CoOwner;
		case CoOwner:
			return Rank.Owner;
		case Owner:
			return Rank.RainbowOwner;
		case RainbowOwner:
			return Rank.Custom;
		case Custom:
			return Rank.ServerOwner;
		case ServerOwner:
			return Rank.ServerHost;
		case ServerHost:
			return Rank.ServerHost;
		}
		return rank;
	}
	
	/**
	 * Return the previous rank on the rank ladder
	 * @param rank The rank
	 * @return The previous rank on the rank ladder
	 */
	public static Rank getPreviousRank(Rank rank) {
		switch (rank) {
		case Builder:
			return Rank.Builder;
		case AdvancedBuilder:
			return Rank.Builder;
		case Architect:
			return Rank.AdvancedBuilder;
		case Moderator:
			return Rank.Architect;
		case Police:
			return Rank.Moderator;
		case Admin:
			return Rank.Police;
		case SpecialAdmin:
			return Rank.Admin;
		case Councillor:
			return Rank.SpecialAdmin;
		case Elite:
			return Rank.Councillor;
		case CoOwner:
			return Rank.Elite;
		case Owner:
			return Rank.CoOwner;
		case RainbowOwner:
			return Rank.Owner;
		case Custom:
			return Rank.RainbowOwner;
		case ServerOwner:
			return Rank.Custom;
		case ServerHost:
			return Rank.ServerOwner;
		}
		return rank;
	}
}
